@@ -6,12 +6,13 @@ import esphome.codegen as cg
 from esphome.components.esp32 import add_idf_sdkconfig_option, const, get_esp32_variant
 import esphome.config_validation as cv
 from esphome.const import CONF_ENABLE_ON_BOOT, CONF_ESPHOME, CONF_ID, CONF_NAME
-from esphome.core import CORE
+from esphome.core import CORE, TimePeriod
 from esphome.core.config import CONF_NAME_ADD_MAC_SUFFIX
 import esphome.final_validate as fv
 
 DEPENDENCIES = ["esp32"]
-CODEOWNERS = ["@jesserockz", "@Rapsssito"]
+CODEOWNERS = ["@jesserockz", "@Rapsssito", "@bdraco"]
+DOMAIN = "esp32_ble"
 
 
 class BTLoggers(Enum):
@@ -115,8 +116,11 @@ def register_bt_logger(*loggers: BTLoggers) -> None:
 
 CONF_BLE_ID = "ble_id"
 CONF_IO_CAPABILITY = "io_capability"
+CONF_ADVERTISING = "advertising"
 CONF_ADVERTISING_CYCLE_TIME = "advertising_cycle_time"
 CONF_DISABLE_BT_LOGS = "disable_bt_logs"
+CONF_CONNECTION_TIMEOUT = "connection_timeout"
+CONF_MAX_NOTIFICATIONS = "max_notifications"
 
 NO_BLUETOOTH_VARIANTS = [const.VARIANT_ESP32S2]
 
@@ -161,12 +165,23 @@ CONFIG_SCHEMA = cv.Schema(
             IO_CAPABILITY, lower=True
         ),
         cv.Optional(CONF_ENABLE_ON_BOOT, default=True): cv.boolean,
+        cv.Optional(CONF_ADVERTISING, default=False): cv.boolean,
         cv.Optional(
             CONF_ADVERTISING_CYCLE_TIME, default="10s"
         ): cv.positive_time_period_milliseconds,
         cv.SplitDefault(CONF_DISABLE_BT_LOGS, esp32_idf=True): cv.All(
             cv.only_with_esp_idf, cv.boolean
         ),
+        cv.SplitDefault(CONF_CONNECTION_TIMEOUT, esp32_idf="20s"): cv.All(
+            cv.only_with_esp_idf,
+            cv.positive_time_period_seconds,
+            cv.Range(min=TimePeriod(seconds=10), max=TimePeriod(seconds=180)),
+        ),
+        cv.SplitDefault(CONF_MAX_NOTIFICATIONS, esp32_idf=12): cv.All(
+            cv.only_with_esp_idf,
+            cv.positive_int,
+            cv.Range(min=1, max=64),
+        ),
     }
 ).extend(cv.COMPONENT_SCHEMA)
 
@@ -255,8 +270,32 @@ async def to_code(config):
                 if logger not in _required_loggers:
                     add_idf_sdkconfig_option(f"{logger.value}_NONE", True)
 
+        # Set BLE connection establishment timeout to match aioesphomeapi/bleak-retry-connector
+        # Default is 20 seconds instead of ESP-IDF's 30 seconds. Because there is no way to
+        # cancel a BLE connection in progress, when aioesphomeapi times out at 20 seconds,
+        # the connection slot remains occupied for the remaining time, preventing new connection
+        # attempts and wasting valuable connection slots.
+        if CONF_CONNECTION_TIMEOUT in config:
+            timeout_seconds = int(config[CONF_CONNECTION_TIMEOUT].total_seconds)
+            add_idf_sdkconfig_option(
+                "CONFIG_BT_BLE_ESTAB_LINK_CONN_TOUT", timeout_seconds
+            )
+
+        # Set the maximum number of notification registrations
+        # This controls how many BLE characteristics can have notifications enabled
+        # across all connections for a single GATT client interface
+        # https://github.com/esphome/issues/issues/6808
+        if CONF_MAX_NOTIFICATIONS in config:
+            add_idf_sdkconfig_option(
+                "CONFIG_BT_GATTC_NOTIF_REG_MAX", config[CONF_MAX_NOTIFICATIONS]
+            )
+
     cg.add_define("USE_ESP32_BLE")
 
+    if config[CONF_ADVERTISING]:
+        cg.add_define("USE_ESP32_BLE_ADVERTISING")
+        cg.add_define("USE_ESP32_BLE_UUID")
+
 
 @automation.register_condition("ble.enabled", BLEEnabledCondition, cv.Schema({}))
 async def ble_enabled_to_code(config, condition_id, template_arg, args):
