@@ -27,15 +27,22 @@ class ESPHomeOTAComponent : public ota::OTAComponent {
   uint16_t get_port() const;
 
  protected:
-  void handle_();
+  void handle_handshake_();
+  void handle_data_();
   bool readall_(uint8_t *buf, size_t len);
   bool writeall_(const uint8_t *buf, size_t len);
+  void log_socket_error_(const char *msg);
+  void log_read_error_(const char *what);
+  void log_start_(const char *phase);
+  void cleanup_connection_();
+  void yield_and_feed_watchdog_();
 
 #ifdef USE_OTA_PASSWORD
   std::string password_;
 #endif  // USE_OTA_PASSWORD
 
   uint16_t port_;
+  uint32_t client_connect_time_{0};
 
   std::unique_ptr<socket::Socket> server_;
   std::unique_ptr<socket::Socket> client_;
