@@ -1,9 +1,8 @@
 import esphome.codegen as cg
-import esphome.config_validation as cv
 from esphome.components import binary_sensor
+import esphome.config_validation as cv
 
-from .. import M5Stack8AngleComponent, m5stack_8angle_ns, CONF_M5STACK_8ANGLE_ID
-
+from .. import CONF_M5STACK_8ANGLE_ID, M5Stack8AngleComponent, m5stack_8angle_ns
 
 M5Stack8AngleSwitchBinarySensor = m5stack_8angle_ns.class_(
     "M5Stack8AngleSwitchBinarySensor",
