@@ -101,12 +101,9 @@ class Application {
     arch_init();
     this->name_add_mac_suffix_ = name_add_mac_suffix;
     if (name_add_mac_suffix) {
-      this->name_ = name + "-" + get_mac_address().substr(6);
-      if (friendly_name.empty()) {
-        this->friendly_name_ = "";
-      } else {
-        this->friendly_name_ = friendly_name + " " + get_mac_address().substr(6);
-      }
+      const std::string mac_suffix = get_mac_address().substr(6);
+      this->name_ = name + "-" + mac_suffix;
+      this->friendly_name_ = friendly_name.empty() ? "" : friendly_name + " " + mac_suffix;
     } else {
       this->name_ = name;
       this->friendly_name_ = friendly_name;
@@ -214,77 +211,6 @@ class Application {
 #endif
 
   /// Reserve space for components to avoid memory fragmentation
-  void reserve_components(size_t count) { this->components_.reserve(count); }
-
-#ifdef USE_BINARY_SENSOR
-  void reserve_binary_sensor(size_t count) { this->binary_sensors_.reserve(count); }
-#endif
-#ifdef USE_SWITCH
-  void reserve_switch(size_t count) { this->switches_.reserve(count); }
-#endif
-#ifdef USE_BUTTON
-  void reserve_button(size_t count) { this->buttons_.reserve(count); }
-#endif
-#ifdef USE_SENSOR
-  void reserve_sensor(size_t count) { this->sensors_.reserve(count); }
-#endif
-#ifdef USE_TEXT_SENSOR
-  void reserve_text_sensor(size_t count) { this->text_sensors_.reserve(count); }
-#endif
-#ifdef USE_FAN
-  void reserve_fan(size_t count) { this->fans_.reserve(count); }
-#endif
-#ifdef USE_COVER
-  void reserve_cover(size_t count) { this->covers_.reserve(count); }
-#endif
-#ifdef USE_CLIMATE
-  void reserve_climate(size_t count) { this->climates_.reserve(count); }
-#endif
-#ifdef USE_LIGHT
-  void reserve_light(size_t count) { this->lights_.reserve(count); }
-#endif
-#ifdef USE_NUMBER
-  void reserve_number(size_t count) { this->numbers_.reserve(count); }
-#endif
-#ifdef USE_DATETIME_DATE
-  void reserve_date(size_t count) { this->dates_.reserve(count); }
-#endif
-#ifdef USE_DATETIME_TIME
-  void reserve_time(size_t count) { this->times_.reserve(count); }
-#endif
-#ifdef USE_DATETIME_DATETIME
-  void reserve_datetime(size_t count) { this->datetimes_.reserve(count); }
-#endif
-#ifdef USE_SELECT
-  void reserve_select(size_t count) { this->selects_.reserve(count); }
-#endif
-#ifdef USE_TEXT
-  void reserve_text(size_t count) { this->texts_.reserve(count); }
-#endif
-#ifdef USE_LOCK
-  void reserve_lock(size_t count) { this->locks_.reserve(count); }
-#endif
-#ifdef USE_VALVE
-  void reserve_valve(size_t count) { this->valves_.reserve(count); }
-#endif
-#ifdef USE_MEDIA_PLAYER
-  void reserve_media_player(size_t count) { this->media_players_.reserve(count); }
-#endif
-#ifdef USE_ALARM_CONTROL_PANEL
-  void reserve_alarm_control_panel(size_t count) { this->alarm_control_panels_.reserve(count); }
-#endif
-#ifdef USE_EVENT
-  void reserve_event(size_t count) { this->events_.reserve(count); }
-#endif
-#ifdef USE_UPDATE
-  void reserve_update(size_t count) { this->updates_.reserve(count); }
-#endif
-#ifdef USE_AREAS
-  void reserve_area(size_t count) { this->areas_.reserve(count); }
-#endif
-#ifdef USE_DEVICES
-  void reserve_device(size_t count) { this->devices_.reserve(count); }
-#endif
 
   /// Register the component in this Application instance.
   template<class C> C *register_component(C *c) {
@@ -368,8 +294,19 @@ class Application {
 
   uint8_t get_app_state() const { return this->app_state_; }
 
-// Helper macro for entity getter method declarations - reduces code duplication
-// When USE_DEVICE_ID is enabled in the future, this can be conditionally compiled to add device_id parameter
+// Helper macro for entity getter method declarations
+#ifdef USE_DEVICES
+#define GET_ENTITY_METHOD(entity_type, entity_name, entities_member) \
+  entity_type *get_##entity_name##_by_key(uint32_t key, uint32_t device_id, bool include_internal = false) { \
+    for (auto *obj : this->entities_member##_) { \
+      if (obj->get_object_id_hash() == key && obj->get_device_id() == device_id && \
+          (include_internal || !obj->is_internal())) \
+        return obj; \
+    } \
+    return nullptr; \
+  }
+  const auto &get_devices() { return this->devices_; }
+#else
 #define GET_ENTITY_METHOD(entity_type, entity_name, entities_member) \
   entity_type *get_##entity_name##_by_key(uint32_t key, bool include_internal = false) { \
     for (auto *obj : this->entities_member##_) { \
@@ -378,100 +315,95 @@ class Application {
     } \
     return nullptr; \
   }
-
-#ifdef USE_DEVICES
-  const std::vector<Device *> &get_devices() { return this->devices_; }
-#endif
+#endif  // USE_DEVICES
 #ifdef USE_AREAS
-  const std::vector<Area *> &get_areas() { return this->areas_; }
+  const auto &get_areas() { return this->areas_; }
 #endif
 #ifdef USE_BINARY_SENSOR
-  const std::vector<binary_sensor::BinarySensor *> &get_binary_sensors() { return this->binary_sensors_; }
+  auto &get_binary_sensors() const { return this->binary_sensors_; }
   GET_ENTITY_METHOD(binary_sensor::BinarySensor, binary_sensor, binary_sensors)
 #endif
 #ifdef USE_SWITCH
-  const std::vector<switch_::Switch *> &get_switches() { return this->switches_; }
+  auto &get_switches() const { return this->switches_; }
   GET_ENTITY_METHOD(switch_::Switch, switch, switches)
 #endif
 #ifdef USE_BUTTON
-  const std::vector<button::Button *> &get_buttons() { return this->buttons_; }
+  auto &get_buttons() const { return this->buttons_; }
   GET_ENTITY_METHOD(button::Button, button, buttons)
 #endif
 #ifdef USE_SENSOR
-  const std::vector<sensor::Sensor *> &get_sensors() { return this->sensors_; }
+  auto &get_sensors() const { return this->sensors_; }
   GET_ENTITY_METHOD(sensor::Sensor, sensor, sensors)
 #endif
 #ifdef USE_TEXT_SENSOR
-  const std::vector<text_sensor::TextSensor *> &get_text_sensors() { return this->text_sensors_; }
+  auto &get_text_sensors() const { return this->text_sensors_; }
   GET_ENTITY_METHOD(text_sensor::TextSensor, text_sensor, text_sensors)
 #endif
 #ifdef USE_FAN
-  const std::vector<fan::Fan *> &get_fans() { return this->fans_; }
+  auto &get_fans() const { return this->fans_; }
   GET_ENTITY_METHOD(fan::Fan, fan, fans)
 #endif
 #ifdef USE_COVER
-  const std::vector<cover::Cover *> &get_covers() { return this->covers_; }
+  auto &get_covers() const { return this->covers_; }
   GET_ENTITY_METHOD(cover::Cover, cover, covers)
 #endif
 #ifdef USE_LIGHT
-  const std::vector<light::LightState *> &get_lights() { return this->lights_; }
+  auto &get_lights() const { return this->lights_; }
   GET_ENTITY_METHOD(light::LightState, light, lights)
 #endif
 #ifdef USE_CLIMATE
-  const std::vector<climate::Climate *> &get_climates() { return this->climates_; }
+  auto &get_climates() const { return this->climates_; }
   GET_ENTITY_METHOD(climate::Climate, climate, climates)
 #endif
 #ifdef USE_NUMBER
-  const std::vector<number::Number *> &get_numbers() { return this->numbers_; }
+  auto &get_numbers() const { return this->numbers_; }
   GET_ENTITY_METHOD(number::Number, number, numbers)
 #endif
 #ifdef USE_DATETIME_DATE
-  const std::vector<datetime::DateEntity *> &get_dates() { return this->dates_; }
+  auto &get_dates() const { return this->dates_; }
   GET_ENTITY_METHOD(datetime::DateEntity, date, dates)
 #endif
 #ifdef USE_DATETIME_TIME
-  const std::vector<datetime::TimeEntity *> &get_times() { return this->times_; }
+  auto &get_times() const { return this->times_; }
   GET_ENTITY_METHOD(datetime::TimeEntity, time, times)
 #endif
 #ifdef USE_DATETIME_DATETIME
-  const std::vector<datetime::DateTimeEntity *> &get_datetimes() { return this->datetimes_; }
+  auto &get_datetimes() const { return this->datetimes_; }
   GET_ENTITY_METHOD(datetime::DateTimeEntity, datetime, datetimes)
 #endif
 #ifdef USE_TEXT
-  const std::vector<text::Text *> &get_texts() { return this->texts_; }
+  auto &get_texts() const { return this->texts_; }
   GET_ENTITY_METHOD(text::Text, text, texts)
 #endif
 #ifdef USE_SELECT
-  const std::vector<select::Select *> &get_selects() { return this->selects_; }
+  auto &get_selects() const { return this->selects_; }
   GET_ENTITY_METHOD(select::Select, select, selects)
 #endif
 #ifdef USE_LOCK
-  const std::vector<lock::Lock *> &get_locks() { return this->locks_; }
+  auto &get_locks() const { return this->locks_; }
   GET_ENTITY_METHOD(lock::Lock, lock, locks)
 #endif
 #ifdef USE_VALVE
-  const std::vector<valve::Valve *> &get_valves() { return this->valves_; }
+  auto &get_valves() const { return this->valves_; }
   GET_ENTITY_METHOD(valve::Valve, valve, valves)
 #endif
 #ifdef USE_MEDIA_PLAYER
-  const std::vector<media_player::MediaPlayer *> &get_media_players() { return this->media_players_; }
+  auto &get_media_players() const { return this->media_players_; }
   GET_ENTITY_METHOD(media_player::MediaPlayer, media_player, media_players)
 #endif
 
 #ifdef USE_ALARM_CONTROL_PANEL
-  const std::vector<alarm_control_panel::AlarmControlPanel *> &get_alarm_control_panels() {
-    return this->alarm_control_panels_;
-  }
+  auto &get_alarm_control_panels() const { return this->alarm_control_panels_; }
   GET_ENTITY_METHOD(alarm_control_panel::AlarmControlPanel, alarm_control_panel, alarm_control_panels)
 #endif
 
 #ifdef USE_EVENT
-  const std::vector<event::Event *> &get_events() { return this->events_; }
+  auto &get_events() const { return this->events_; }
   GET_ENTITY_METHOD(event::Event, event, events)
 #endif
 
 #ifdef USE_UPDATE
-  const std::vector<update::UpdateEntity *> &get_updates() { return this->updates_; }
+  auto &get_updates() const { return this->updates_; }
   GET_ENTITY_METHOD(update::UpdateEntity, update, updates)
 #endif
 
@@ -519,12 +451,7 @@ class Application {
   const char *comment_{nullptr};
   const char *compilation_time_{nullptr};
 
-  // size_t members
-  size_t dump_config_at_{SIZE_MAX};
-
-  // Vectors (largest members)
-  std::vector<Component *> components_{};
-
+  // std::vector (3 pointers each: begin, end, capacity)
   // Partitioned vector design for looping components
   // =================================================
   // Components are partitioned into [active | inactive] sections:
@@ -542,85 +469,17 @@ class Application {
   //   and active_end_ is incremented
   // - This eliminates branch mispredictions from flag checking in the hot loop
   std::vector<Component *> looping_components_{};
-
-#ifdef USE_DEVICES
-  std::vector<Device *> devices_{};
-#endif
-#ifdef USE_AREAS
-  std::vector<Area *> areas_{};
-#endif
-#ifdef USE_BINARY_SENSOR
-  std::vector<binary_sensor::BinarySensor *> binary_sensors_{};
-#endif
-#ifdef USE_SWITCH
-  std::vector<switch_::Switch *> switches_{};
-#endif
-#ifdef USE_BUTTON
-  std::vector<button::Button *> buttons_{};
-#endif
-#ifdef USE_EVENT
-  std::vector<event::Event *> events_{};
-#endif
-#ifdef USE_SENSOR
-  std::vector<sensor::Sensor *> sensors_{};
-#endif
-#ifdef USE_TEXT_SENSOR
-  std::vector<text_sensor::TextSensor *> text_sensors_{};
-#endif
-#ifdef USE_FAN
-  std::vector<fan::Fan *> fans_{};
-#endif
-#ifdef USE_COVER
-  std::vector<cover::Cover *> covers_{};
-#endif
-#ifdef USE_CLIMATE
-  std::vector<climate::Climate *> climates_{};
-#endif
-#ifdef USE_LIGHT
-  std::vector<light::LightState *> lights_{};
-#endif
-#ifdef USE_NUMBER
-  std::vector<number::Number *> numbers_{};
-#endif
-#ifdef USE_DATETIME_DATE
-  std::vector<datetime::DateEntity *> dates_{};
-#endif
-#ifdef USE_DATETIME_TIME
-  std::vector<datetime::TimeEntity *> times_{};
-#endif
-#ifdef USE_DATETIME_DATETIME
-  std::vector<datetime::DateTimeEntity *> datetimes_{};
-#endif
-#ifdef USE_SELECT
-  std::vector<select::Select *> selects_{};
-#endif
-#ifdef USE_TEXT
-  std::vector<text::Text *> texts_{};
-#endif
-#ifdef USE_LOCK
-  std::vector<lock::Lock *> locks_{};
-#endif
-#ifdef USE_VALVE
-  std::vector<valve::Valve *> valves_{};
-#endif
-#ifdef USE_MEDIA_PLAYER
-  std::vector<media_player::MediaPlayer *> media_players_{};
-#endif
-#ifdef USE_ALARM_CONTROL_PANEL
-  std::vector<alarm_control_panel::AlarmControlPanel *> alarm_control_panels_{};
-#endif
-#ifdef USE_UPDATE
-  std::vector<update::UpdateEntity *> updates_{};
-#endif
-
 #ifdef USE_SOCKET_SELECT_SUPPORT
   std::vector<int> socket_fds_;  // Vector of all monitored socket file descriptors
 #endif
 
-  // String members
+  // std::string members (typically 24-32 bytes each)
   std::string name_;
   std::string friendly_name_;
 
+  // size_t members
+  size_t dump_config_at_{SIZE_MAX};
+
   // 4-byte members
   uint32_t last_loop_{0};
   uint32_t loop_component_start_time_{0};
@@ -630,9 +489,9 @@ class Application {
 #endif
 
   // 2-byte members (grouped together for alignment)
-  uint16_t loop_interval_{16};  // Loop interval in ms (max 65535ms = 65.5 seconds)
-  uint16_t looping_components_active_end_{0};
-  uint16_t current_loop_index_{0};  // For safe reentrant modifications during iteration
+  uint16_t loop_interval_{16};                 // Loop interval in ms (max 65535ms = 65.5 seconds)
+  uint16_t looping_components_active_end_{0};  // Index marking end of active components in looping_components_
+  uint16_t current_loop_index_{0};             // For safe reentrant modifications during iteration
 
   // 1-byte members (grouped together to minimize padding)
   uint8_t app_state_{0};
@@ -642,11 +501,87 @@ class Application {
 
 #ifdef USE_SOCKET_SELECT_SUPPORT
   bool socket_fds_changed_{false};  // Flag to rebuild base_read_fds_ when socket_fds_ changes
+#endif
 
-  // Variable-sized members at end
+#ifdef USE_SOCKET_SELECT_SUPPORT
+  // Variable-sized members
   fd_set base_read_fds_{};  // Cached fd_set rebuilt only when socket_fds_ changes
   fd_set read_fds_{};       // Working fd_set for select(), copied from base_read_fds_
 #endif
+
+  // StaticVectors (largest members - contain actual array data inline)
+  StaticVector<Component *, ESPHOME_COMPONENT_COUNT> components_{};
+
+#ifdef USE_DEVICES
+  StaticVector<Device *, ESPHOME_DEVICE_COUNT> devices_{};
+#endif
+#ifdef USE_AREAS
+  StaticVector<Area *, ESPHOME_AREA_COUNT> areas_{};
+#endif
+#ifdef USE_BINARY_SENSOR
+  StaticVector<binary_sensor::BinarySensor *, ESPHOME_ENTITY_BINARY_SENSOR_COUNT> binary_sensors_{};
+#endif
+#ifdef USE_SWITCH
+  StaticVector<switch_::Switch *, ESPHOME_ENTITY_SWITCH_COUNT> switches_{};
+#endif
+#ifdef USE_BUTTON
+  StaticVector<button::Button *, ESPHOME_ENTITY_BUTTON_COUNT> buttons_{};
+#endif
+#ifdef USE_EVENT
+  StaticVector<event::Event *, ESPHOME_ENTITY_EVENT_COUNT> events_{};
+#endif
+#ifdef USE_SENSOR
+  StaticVector<sensor::Sensor *, ESPHOME_ENTITY_SENSOR_COUNT> sensors_{};
+#endif
+#ifdef USE_TEXT_SENSOR
+  StaticVector<text_sensor::TextSensor *, ESPHOME_ENTITY_TEXT_SENSOR_COUNT> text_sensors_{};
+#endif
+#ifdef USE_FAN
+  StaticVector<fan::Fan *, ESPHOME_ENTITY_FAN_COUNT> fans_{};
+#endif
+#ifdef USE_COVER
+  StaticVector<cover::Cover *, ESPHOME_ENTITY_COVER_COUNT> covers_{};
+#endif
+#ifdef USE_CLIMATE
+  StaticVector<climate::Climate *, ESPHOME_ENTITY_CLIMATE_COUNT> climates_{};
+#endif
+#ifdef USE_LIGHT
+  StaticVector<light::LightState *, ESPHOME_ENTITY_LIGHT_COUNT> lights_{};
+#endif
+#ifdef USE_NUMBER
+  StaticVector<number::Number *, ESPHOME_ENTITY_NUMBER_COUNT> numbers_{};
+#endif
+#ifdef USE_DATETIME_DATE
+  StaticVector<datetime::DateEntity *, ESPHOME_ENTITY_DATE_COUNT> dates_{};
+#endif
+#ifdef USE_DATETIME_TIME
+  StaticVector<datetime::TimeEntity *, ESPHOME_ENTITY_TIME_COUNT> times_{};
+#endif
+#ifdef USE_DATETIME_DATETIME
+  StaticVector<datetime::DateTimeEntity *, ESPHOME_ENTITY_DATETIME_COUNT> datetimes_{};
+#endif
+#ifdef USE_SELECT
+  StaticVector<select::Select *, ESPHOME_ENTITY_SELECT_COUNT> selects_{};
+#endif
+#ifdef USE_TEXT
+  StaticVector<text::Text *, ESPHOME_ENTITY_TEXT_COUNT> texts_{};
+#endif
+#ifdef USE_LOCK
+  StaticVector<lock::Lock *, ESPHOME_ENTITY_LOCK_COUNT> locks_{};
+#endif
+#ifdef USE_VALVE
+  StaticVector<valve::Valve *, ESPHOME_ENTITY_VALVE_COUNT> valves_{};
+#endif
+#ifdef USE_MEDIA_PLAYER
+  StaticVector<media_player::MediaPlayer *, ESPHOME_ENTITY_MEDIA_PLAYER_COUNT> media_players_{};
+#endif
+#ifdef USE_ALARM_CONTROL_PANEL
+  StaticVector<alarm_control_panel::AlarmControlPanel *, ESPHOME_ENTITY_ALARM_CONTROL_PANEL_COUNT>
+      alarm_control_panels_{};
+#endif
+#ifdef USE_UPDATE
+  StaticVector<update::UpdateEntity *, ESPHOME_ENTITY_UPDATE_COUNT> updates_{};
+#endif
 };
 
 /// Global storage of Application pointer - only one Application can exist.
